const { promisify } = require('util');
const { dirname, join } = require('path');
const { copyFile, mkdir, stat } = require('fs');
const { totalist } = require('totalist');

const pCopyFile = promisify(copyFile)
const pMkdir = promisify(mkdir)
const pStat = promisify(stat)

const mergefs = async ({ input, output, files }) => {
	const fileMap = {}

	for (const inputFolder of (input || [])) {
		if (files) {
			for (const singleFile of files) {
				try {
					const stats = await pStat(join(inputFolder, singleFile))
					if (!stats.isDirectory()) {
						fileMap[singleFile] = inputFolder
					}
				} catch (ignore) {}
			}
		} else {
			await totalist(inputFolder, name => {
				fileMap[name] = inputFolder
			})
		}
	}

	for (const [ file, inputFolder ] of Object.entries(fileMap)) {
		await pMkdir(dirname(join(output, file)), { recursive: true })
		await pCopyFile(join(inputFolder, file), join(output, file))
	}
}

exports.mergefs = mergefs;